/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Pavol Vaskovic.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Andy Edmonds <aedmonds@mindspring.com>
 *  Benjamin K. Stuhl <tiriath@yahoo.com>
 *  Brent Schartung <bschartung@gmail.com>
 *  David Illsley <illsleydc@bigfoot.com>
 *  Dorando <mozilla@dorando.at>
 *  HJ van Rantwijk <bugs4HJ@netscape.net>
 *  Jens Bannmann <jens.b@web.de>
 *  Jochen <autohide@krickelkrackel.de>
 *  Martin.T.Kutschker <Martin.T.Kutschker@blackbox.net>
 *  Pavol Vaskovic <pali@pali.sk>
 *  Pekka Aleksi Knuutila <aleksi.knuutila@edu.lahti.fi>
 *  Scott R. Turner <srt@aero.org>
 *  Tim Williamson <chsman@hotmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//----------------------
//-- Global variables --
//----------------------

var mgContext = null;
var mgContent = null;
var mgGetAutoScroll;


//-----------------
//-- Preferences --
//-----------------

var mgButton = -1;
var mgLeftHanded = false;
// mouse gesture condition (to be evaluated on event) for more details see mozgestReadPref()
var mgGestureCondition = new String;
var grid; // grid size; minimal gesture has to be 'grid' pixels long
var delay; // delay before aborting gesture
var lmbGestureLimit;
var dragdropDelay;
var rockersEnabled;
var mgWheelRockers;
var mgWheelSensitivity;
var mgSideBarSort;
var dTolerance; // tolerance for diagonal movement. see processCoordinates()

// trail settings
var drawTrail;


//---------------------
//-- State variables --
//---------------------

var gestureStartTime;
var mgLastGestureTime; // end timestamp of the last gesture attempt
var gestureInProgress = false;
var mgGestureDone = false;
var mgRockerAborted = false;
var mgPreviousSelection = null;
// old coordinates from previous gesture stroke used in processCoordinates()
var mgOldX;
var mgOldY;
var mgStartX = 0;
var mgStartY = 0;
var gridMoves = new Array();
var gesture = new Array();
var localizedGesture = new Array();
var mgString; // array holding localized stroke names
var lastgesture;
// lastGestureTime is time of last gesture stroke, compared with delay and current time
var lastGestureTime;
var lastMoveTime;
// timer used to abort gestures after delay, see processCoordinates()
var mgGestureTimeout;
var mgAllowContextMenu = false;
var mgAutoScrollAborted = false;

var globalOnLink = false; // array that holds a list of all the links traversed during gesture
var globalOnImage = false; // string containing an image href or false
var globalSrcEvent = false; // event which started the active gesture.
var mgLastHoveredElement = null;


//----------------
//-- Components --
//----------------

// used by gesture implementations
const mgkWindowMediator = Components
      .classes["@mozilla.org/appshell/window-mediator;1"]
      .getService(Components.interfaces.nsIWindowMediator);
var mgPrefRoot = null;

// other components
var mgPref;
var mgBundle = null; // String bundle for localized strings


//--------------------
//-- Observer class --
//--------------------

function mgObserver()
{
  this.register();
}
mgObserver.prototype = {
  observe: function(subject, topic, data) {
    // when MozGest pref was changed, we reinitialize (except sidebar)
    if (topic.indexOf("nsPref") == 0 &&  data != "mozgest.sideBarSort") {
      mozgestInit();
    }
    if (topic == "mozgestControl") {
      if (data == "shutdown") {
        window.setTimeout(mgShutdown, 0);
      } else if (data == "wasUninstalled") {
        // If we're in firefox and our sidebar is open, close it. Otherwise,
        // a phantom sidebar will always stay open.
        var sidebarBox = document.getElementById("sidebar-box");
        if (sidebarBox != null
            && sidebarBox.getAttribute("sidebarcommand") == "viewMozgestSidebar") {
          toggleSidebar();
          // re-focus the EM window
          BrowserOpenExtensions("extensions");
        }
      }
    }
  },
  register: function() {
    try {
      mgPref.QueryInterface(Components.interfaces.nsIPrefBranchInternal)
            .addObserver("", this, false);

      var observerService = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
      observerService.addObserver(this, "mozgestControl", false);
    } catch(ex) {
      dump("MozGest: Failed to initialize observer: " + ex + "\n");
    }
  },
  unregister: function() {
    mgPref.QueryInterface(Components.interfaces.nsIPrefBranchInternal)
          .removeObserver("", this);

    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(this, "mozgestControl");
  }
}


//--------------------------------
//-- Startup/shutdown functions --
//--------------------------------

function mgStartup(e) {
  // Delay the removal of our handler, so it occurs after this event phase.
  // This avoids a bug that skips other listeners, see
  // http://bugzilla.mozilla.org/show_bug.cgi?id=174320
  setTimeout("window.removeEventListener('load', mgStartup, false);",0);

  determineWindowType();
  if (windowType) {
    dump("MozGest: ---- Startup ----\n");
    dockElement("stringbundleset", "stringbundle", "mozgestbundle",
        new Array("src"),
        new Array("chrome://mozgest/locale/mozgest.properties"));
    dockElement("keyset", "key", "mgPrefKey",
        new Array("modifiers",   "key", "command"  ),
        new Array("accel shift", "M",   "mgPrefCmd"));
    dockElement("commandset", "command", "mgPrefCmd",
        new Array("oncommand"),
        new Array("showMgPrefPanel();"));

    // get the preferences service
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
    mgPrefRoot = prefService.getBranch(null); // root preferences branch
    mgPref = prefService.getBranch("mozgest."); // mozgest preferences branch

    // init our observer
    window.mozgestObserver = new mgObserver();

    window.addEventListener("unload", mgShutdown, false);
    mozgestLoadStrings();
    mozgestInit();
  }
}

function mgShutdown() {
  dump("MozGest: ---- Shutdown ----\n");
  window.mozgestObserver.unregister();
  removeWindowWatch();
  undockElement("mozgestbundle");
  undockElement("mgPrefKey");
  undockElement("mgPrefCmd");
  mgStorage.shutdown();

  // Remove this event listener after all other listeners have completed
  window.setTimeout("window.removeEventListener(\"unload\", mgShutdown, false);", 0);
}

function mozgestInit() {
  // read prefs (or set default for each missing setting)
  mozgestReadPref();

  // turn gestures recognition on/off
  if (mgIsEnabled()) {
    addWindowWatch();
    loadGestureTable();
  } else {
    removeWindowWatch();
  }
}

function loadGestureTable(aBool) {
  dump("Loading gesture table\n");
  resetGesturesTable();
  mgStorage.initUserRDF();
  mgStorage.readRDFGestureTable("window");
  mgStorage.readRDFGestureTable(windowType);

  // notify the sidebar(s), do this only once if there is more than one window
  if (aBool && mgkWindowMediator.getMostRecentWindow("navigator:browser") == window) {
     Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService)
               .notifyObservers(null, "mozgestControl", "mappingsUpdated");
  }
}

function mozgestLoadStrings() {
  // load localized string bundle and cache often used strings
  mgBundle = document.getElementById("mozgestbundle");
  mgString = new Array();
  mgString["R"] = mgBundle.getString("abbreviation.right");
  mgString["L"] = mgBundle.getString("abbreviation.left");
  mgString["U"] = mgBundle.getString("abbreviation.up");
  mgString["D"] = mgBundle.getString("abbreviation.down");
  mgString["1"] = mgBundle.getString("abbreviation.d1");
  mgString["3"] = mgBundle.getString("abbreviation.d3");
  mgString["7"] = mgBundle.getString("abbreviation.d7");
  mgString["9"] = mgBundle.getString("abbreviation.d9");
  mgString["gesture"] = mgBundle.getString("g.gesture");
}

function addWindowWatch() {
  dump("MozGest: ---- Get Contentarea ---\n");
  if (document.documentElement.getAttribute("windowtype")=="navigator:browser")
  {  mgContent = mgGetContentArea().mPanelContainer;
     mgGetAutoScroll = "getBrowser().mCurrentBrowser"
  }
  else
  {  mgContent = mgGetContentArea();
     mgGetAutoScroll = "mgGetContentArea()"
  }

  removeWindowWatch();
  dump("MozGest: Adding Listeners\n");
  mgContent.addEventListener("mousedown", startGesture, false);
  mgContent.addEventListener("draggesture", onDragGestureEvent, true);
  if (mgWheelRockers && rockersEnabled) {
    mgContent.addEventListener("DOMMouseScroll", mgMousewheelHandler, false);
  }
  window.addEventListener("mouseup", endGesture, false);
  window.addEventListener("keydown", mgAllowContextByKeyPress, true);
  window.addEventListener("mousemove", processCoordinates, false);
  mgContent.addEventListener("contextmenu", mgContextMenuListener, true);
  mgContent.addEventListener("click", mgOnClickHandler, true);
  mgContent.addEventListener("dblclick", mgOnClickHandler, true);
}

function removeWindowWatch() {
  dump("MozGest: Removing Listeners\n");
  mgContent.removeEventListener("mousedown", startGesture, false);
  mgContent.removeEventListener("draggesture", onDragGestureEvent, true);
  mgContent.removeEventListener("DOMMouseScroll", mgMousewheelHandler, false)
  mgContent.removeEventListener("contextmenu", mgContextMenuListener, true)
  window.removeEventListener("mouseup", endGesture, false);
  window.removeEventListener("keydown", mgAllowContextByKeyPress, true);
  window.removeEventListener("mousemove", processCoordinates, false);
  mgContent.removeEventListener("click", mgOnClickHandler, true);
  mgContent.removeEventListener("dblclick", mgOnClickHandler, true);
}


//-----------------------
//-- Integration hooks --
//-----------------------
// (everything needed for co-existing with or disabling regular app functions)

function mgContextMenuListener() {
    addEventListener("popupshowing", mgDisableContextMenu, true);
    setTimeout("removeEventListener('popupshowing', mgDisableContextMenu, true)",0);
}

function mgDisableContextMenu(e) {
    mgContext=e.originalTarget;
    if (!mgAllowContextMenu)
       e.preventDefault();
}

function mgAllowContextByKeyPress(e) {
    mgAllowContextMenu = true;
    mgGestureDone = false;
}

function mgOnClickHandler(e) {
  var now = (new Date).getTime();
  if ((e.button == mgButton && now - mgLastGestureTime <= 1) || mgGestureDone) {
    dump("MozGest: Stopping " + e.type + " event caused by last gesture.\n");
    e.preventDefault();
    e.stopPropagation();
    return false;
  }

  var node = e.originalTarget;
  if (!node.getAttribute) return null;
  if (!node.hasAttribute("href")) return null;
  var tagname = node.nodeName.toLowerCase();
  var href = node.getAttribute("href");
  if (tagname == "a" && href.indexOf("mozgest://") == 0) {
    dump("MozGest: Link click " + href + "\n");
    mgStorage.processURL(href);
    e.stopPropagation();
    e.preventDefault();
  }
  return null;
}

function onDragGestureEvent(e) {
  var draggedElement = e.originalTarget.nodeName.toUpperCase();
  if (gestureInProgress && draggedElement != "HTML") {
    if (lastMoveTime-gestureStartTime > dragdropDelay) {
      dump("MozGest: '" + draggedElement
           + "' drag encountered, canceling gesture\n");
      endGesture("drag");
    } else {
      dump("MozGest: canceling '" + draggedElement
           + "' drag event, continuing gesture\n");
      e.stopPropagation();
    }
  }
  if (mgGestureDone)
     e.stopPropagation();
}


//---------------------------------
//-- The gesture recognizer core --
//---------------------------------

function processCoordinates(e){ // e is browser's event object
  if (e.screenX < mgStartX -25 || e.screenX > mgStartX +25 ||
      e.screenY < mgStartY -25 || e.screenY > mgStartY +25) {
    mgRockerAborted = true;
  }

  if (!gestureInProgress) return;

  if (mgButton == 1 && !mgAutoScrollAborted) {
     mgAutoScrollAborted = true;
     if (eval(mgGetAutoScroll).autoscrollEnabled) {
        eval(mgGetAutoScroll).stopScroll();
        return;
     }
     if ( ("isScrolling" in window) ) {
        stopScroll();
        globalOnImage = false;
        return;
     }
  }

  if (drawTrail && !mgTrails.noContent && mgTrails.trailLength < mgTrails.limit)
     mgTrails.draw(e.screenX, e.screenY, e);

  var g = gesture.join("");
  if (mgGestureCondition == "e.button==0" && (g == "L" || g == "R")
      && (e.timeStamp - gestureStartTime) > lmbGestureLimit) {
    endGesture(null);
  }

  var x_dir = e.clientX - mgOldX;
  var y_dir = e.clientY - mgOldY;
  var x = Math.abs(x_dir);
  var y = Math.abs(y_dir);

  if ( (x >= grid/2 || y >= grid/2) ) { // process each half-grid
    // diagonal movement:
    // dTolerance = 75 means that a movement is recognized as diagonal when
    // x/y or y/x is between 0.25 and 1
    if ( dTolerance != 0 && ( (x/y >= (1-dTolerance/100) && x/y <= 1)
        || (y/x >= (1-dTolerance/100) && y/x <= 1) ) ) {

      if (x_dir < 0 && y_dir > 0) {
        gPush("1");
      } else if (x_dir > 0 && y_dir > 0) {
        gPush("3");
      } else if (x_dir < 0 && y_dir < 0) {
        gPush("7");
      } else if (x_dir > 0 && y_dir < 0) {
        gPush("9");
      }

    }
    // horizontal movement:
    else if (x > y) {
      if (x_dir > 0) {
        gPush("R");
      } else if (x_dir < 0) {
        gPush("L");
      }
    }
    // vertical movement:
    else if (x < y) {
      if (y_dir > 0){
        gPush("D");
      } else if (y_dir < 0){
        gPush("U");
      }
    }
    var statusText = mgString["gesture"] + " " + localizedGesture;
    var mapping = gesturesTable[gesture.join('')];
    if(mapping != null) {
        statusText += " (" + mapping.name + ")";
    }
    mgMsg(statusText);
//    mgMsg(mgString["gesture"] + " " + localizedGesture);
    lastGestureTime = e.timeStamp;
    if (mgGestureTimeout) { window.clearTimeout(mgGestureTimeout); }
    mgGestureTimeout = window.setTimeout("endGesture(null);", delay);

    mgOldX = e.clientX;
    mgOldY = e.clientY;
  }
  lastMoveTime = e.timeStamp;

  mgFindHoveredElements(e.originalTarget);
}

function gPush(code) {
  gridMoves.push(code);
  if (gridMoves.length == 2) {
    if (gridMoves[0] == gridMoves[1] || gesture.length == 0) {
      doPush(gridMoves.pop());
    }
    gridMoves.length = 0;
  }
}

function doPush(code) {
  if (gesture[gesture.length-1] != code) {
    gesture.push(code);
    localizedGesture.push(mgString[code]);
  }
  if (gesture.length == 1) resetRocker();
}

function mgFindHoveredElements(domPosition) {
  var distance = 0;
  while (domPosition != null
         && domPosition != mgContent
         && distance < 5) {
    mgExamineHoveredElement(domPosition);
    domPosition = domPosition.parentNode;
    distance++;
  }
}

function mgExamineHoveredElement(node) {
  var tag = node.nodeName.toLowerCase();
  if (tag == "a" && node.href && !mozgestCheckLinkDups(node.href)) {
    if(!globalOnLink) globalOnLink = new Array();
    globalOnLink.push(node);
    dump("Mozgest: hovered link '" + node.href + "'\n");
  } else if (tag == "img" && globalOnImage == false) {
    globalOnImage = node;
    dump("Mozgest: found image.\n");
  }
}

function mozgestCheckLinkDups(addy) { // avoid duplicates in globalOnLink array
  var isADup = false;
  for(var zz = 0; zz < globalOnLink.length; zz++) {
    if(globalOnLink[zz].href == addy) isADup = true;
  }
  return isADup;
}

// startGesture is called from event listener in bubble phase
function startGesture(e){
  mgAllowContextMenu = false;
  mgGestureDone = false;
  mgStartX = e.screenX;
  mgStartY = e.screenY;

  // for Autoscroll by skidooer
  if (e.button == 1 && ("_autoScrollMarkerImage" in window) && _autoScrollMarkerImage)
     _autoScrollMarkerImage.addEventListener("mousedown", startGesture, false);

  // collect ancestor elements, which are needed by inContent checks
  var ancestors = new Array();
  var cur = e.originalTarget;
  while (cur != null && cur != mgContent) {
    var tag = cur.nodeName.toLowerCase();
    if (!ancestors[tag]) ancestors[tag] = cur;
    cur = cur.parentNode;
  }

  if (mozgestIsInContent(ancestors)) {
    globalOnLink = false;
    globalOnImage = false;
    globalSrcEvent = e;

    // check if we started over link or image
    if (ancestors["a"]) mgExamineHoveredElement(ancestors["a"]);
    if (ancestors["img"]) mgExamineHoveredElement(ancestors["img"]);

    if(!checkForRocker(e) // note: rockers *can* start while gestureInProgress
       && !gestureInProgress
       && eval(mgGestureCondition)) {

      // check if it's possible to draw trails
      if (drawTrail)
         mgTrails.shouldDrawTrail(e);

      gestureInProgress = true;
      gestureStartTime = e.timeStamp;
      lastMoveTime = 0;
      mgOldX = e.clientX;
      mgOldY = e.clientY;

      var sel = mgGetSelection();
      if (mgButton == 0 && sel.rangeCount > 0)
      {
        mgPreviousSelection = sel.getRangeAt(0);
      }

      gesture.length = 0; // clear gesture array
      localizedGesture.length = 0; // clear localized gesture array
      lastGestureTime = e.timeStamp;
    }
  }
}

function endGesture(e) {
  releaseRocker(e);
  mgRockerAborted = false;
  mgAutoScrollAborted = false;

  if (gestureInProgress) {
    if (mgButton == 0)
       window.setTimeout("mgTrails.endTrail()", 1);
    else
       mgTrails.endTrail();
    window.clearTimeout(mgGestureTimeout);
    lastgesture = gesture.join("");

    // check if we've been called via a timeout; see processCoordinates()
    if (e == null) {
      dump("gesture timeouted\n");
      mgMsg(mgBundle.getString("g.aborted"));
      mgGestureDone = true;
    } else if (e == "drag") {
    } else if (lastgesture != ""){
      window.setTimeout(fireGesture, 1, lastgesture);
      mgGestureDone = true;

      // we need to save the current timestamp here, so if a click event
      // follows, it can be detected as related to this mouseup event
      // (see mgOnClickHandler)
      mgLastGestureTime = (new Date).getTime();
    } else {
      dump("MozGest: Gesture was too small\n");
    }

    gestureInProgress = false;
    gestureStartTime = false;
  }

  if (e && !mgGestureDone && (e.button == 2 || (navigator.platform == "MacPPC"
                                               && e.button == 0 && e.ctrlKey)))
  {
     mgAllowContextMenu = true;
                           // contextmenu on mousedown
     if (navigator.platform != "Win32" && navigator.platform != "OS/2") {
        document.popupNode = e.target;
        var mgContentBox = mgContent.boxObject
        if (e.screenY >= mgContentBox.screenY &&
            e.screenY <= mgContentBox.screenY + mgContentBox.height &&
            e.screenX >= mgContentBox.screenX &&
            e.screenX <= mgContentBox.screenX + mgContentBox.width)

            mgContext.showPopup(mgContext.ownerDocument.documentElement, e.clientX, e.clientY,
                            "context", "bottomleft", "topleft");
     }                    // end contextmenu on mousedown
  }
}


//---------------------------
//-- Preferences interface --
//---------------------------

function mozgestReadPref(){ // read settings from Preferences interface
  var isMac = navigator.platform == "MacPPC";

  grid = mgGetPref("grid", "Int", 15, 4, 500);
  delay = mgGetPref("delay", "Int", 1000, 200, 10000);
  lmbGestureLimit = mgGetPref("lmbGestureLimit", "Int", 500, 100, 10000);
  dragdropDelay = mgGetPref("dragdropDelay", "Int", 400, 100, 10000);
  rockersEnabled = mgGetPref("enableRockers", "Bool", true);
  mgWheelRockers = mgGetPref("wheelRockers", "Bool", false);
  mgWheelSensitivity = mgGetPref("wheelSensitivity", "Int", 1, 1, 9);
  dTolerance = mgGetPref("diagonalTolerance", "Int", 60, 0, 75);

  mgButton = mgGetPref("mousebutton", "Int", (isMac ? 0 : 2), 0, 2);
  mgLeftHanded = mgGetPref("lefthanded", "Bool", false);
  if (mgLeftHanded)
    mgButton = Math.abs(mgButton - 2);

  // mgGestureCondition is a boolean expression consisting of event object values
  mgGestureCondition = "e.button==" + mgButton;
  if (mgGetPref("modifier.ctrl", "Bool", false))
    mgGestureCondition += " && e.ctrlKey";
  if (mgGetPref("modifier.alt", "Bool", false))
    mgGestureCondition += " && e.altKey";
  if (mgGetPref("modifier.shift", "Bool", false))
    mgGestureCondition += " && e.shiftKey";
  if (isMac && mgGetPref("modifier.meta", "Bool", false))
    mgGestureCondition += " && e.metaKey";

  drawTrail = mgGetPref("trails.enabled", "Bool", false);
  mgTrails.width = mgGetPref("trails.width", "Int", 5, 1, 100);
  mgTrails.limit = mgGetPref("trails.length-limit", "Int", 1000, 1, 10000);
  mgTrails.interval = mgGetPref("trails.interval", "Int", 2, 1, 100);
  mgTrails.color = mgGetPref("trails.color", "Char", "#FF0000");
  mgTrails.opacity = mgGetPref("trails.opacity", "Int", 100, 1, 100);

  if (navigator.productSub > 20040113 && !("@mozilla.org/browser/fullscreen;1" in Components.classes))
     mgTrails.older = false;
  else
     mgTrails.older = true;

  mgSideBarSort = mgGetPref("sideBarSort", "Char", "sortByUsage");
}

// Gets a pref setting. If it's missing, set it to a default value.
// For Int prefs, you can optionally specify a minimum and maximum value.
function mgGetPref(name, type, defaultValue, min, max) {
  if ((type != "Bool" && type != "Char" && type != "Int") || !name) {
    return null;
  }
  var notfound = false;
  var value;
  try {
    value = eval("mgPref.get" + type + "Pref(\"" + name + "\");");
  } catch (e) {
    notfound = true;
  }
  if (notfound || (type == "Int" && (min || max) && (value > max || value < min))) {
    val = (type == "Char") ? ("\"" + defaultValue + "\"") : defaultValue;
    eval("mgPref.set" + type + "Pref(\"" + name + "\", " + val + ");");
    return defaultValue;
  } else {
    return value;
  }
}


//-----------------------
//-- Utility functions --
//-----------------------

function showMgPrefPanel() {
  window.openDialog("chrome://mozgest/content/pref/mozgestPrefWindow.xul",
                    "mozgestprefs", "chrome,centerscreen,modal");
}

function dockElement(setType, type, id, aNames, aValues) {
  dockElementAt(setType, type, id, aNames, aValues, document.documentElement);
}

function dockElementAt(setType, type, id, aNames, aValues, container) {
  var sets = document.getElementsByTagName(setType);
  var set;
  if (sets && sets.length > 0) {
    set = sets[0];
  } else {
    set = document.createElement(setType);
    container.appendChild(set);
  }
  var element = document.createElement(type);
  element.setAttribute("id", id);
  for (i=0; i < aNames.length; i++) {
    element.setAttribute(aNames[i], aValues[i]);
  }
  set.appendChild(element);
}

function undockElement(id) {
  var node = document.getElementById(id);
  node.parentNode.removeChild(node);
}

function mgGetBuildProperties() {
  return mgGetBundle("chrome://mozgest/content/mgbuild.properties");
}

function mgGetBundle(url) {
  return Components.classes["@mozilla.org/intl/stringbundle;1"].getService()
                   .QueryInterface(Components.interfaces.nsIStringBundleService)
                   .createBundle(url);
}